# -*- coding: UTF-8 -*-
import pyuc
from pyuc.py_api_b import PyApiB
import time
if PyApiB.tryImportModule("adbutils",installName="adbutils"):
    from adbutils import adb,AdbDevice


class AdbU(PyApiB):
    """
    adb相关工具
    """
    @staticmethod
    def produce(key=None):
        return PyApiB._produce(key, __class__)
    
    def __init__(self):
        self.asyncU = pyuc.asyncU()
        self.devices = {}

    def devices(self):
        """ 获取所有可用的设备 """
        return adb.devices()

    def device(self, serial=None)->AdbDevice:
        """ 获取设备,当只有一个设备时，可以不填serial """
        d = self.devices.get(str(serial))
        if d == None:
            d = adb.device(serial)
            self.devices[str(serial)] = d
        return self.devices[str(serial)]

    def screenshot(self, savePath='screen.jpg', serial=None):
        d = self.device(serial)
        remote_tmp_path = "/data/local/tmp/screenshot.png"
        d.shell(["rm", remote_tmp_path])
        d.shell(["screencap", "-p", remote_tmp_path])
        d.sync.pull(remote_tmp_path, savePath)

    def click(self, x, y, serial=None, times=1):
        if times <= 0:
            return
        elif times == 1:
            d = self.device(serial)
            d.click(x, y)
        else:
            self.asyncU.asyncRun(target=self.click,args=(x,y,serial,times-1))
            time.sleep(0.2)
            
    def shell(self, cmdLine, serial=None):
        d = self.device(serial)
        d.shell(cmdLine.split(" "))
        
    def swipe(self, x1, y1, x2, y2, ss,serial=None):
        """ swipe from(10, 10) to(200, 200) 0.5s """
        d = self.device(serial)
        d.swipe(x1, y1, x2, y2, ss)
        
    def window_size(self, serial=None):
        d = self.device(serial)
        return d.window_size()
    
    def rotation(self, serial=None):
        d = self.device(serial)
        return d.rotation()
    
    def keyHome(self, serial=None):
        self.keyEvent("HOME",serial)
    
    def keyBack(self, serial=None):
        self.keyEvent("BACK",serial)
        
    def keyEvent(self, keyEvent, serial=None):
        d = self.device(serial)
        d.keyevent(keyEvent)
        
    def send_keys(self, keys, serial=None):
        d = self.device(serial)
        d.send_keys(keys)
        
    def is_screen_on(self, serial=None):
        d = self.device(serial)
        d.is_screen_on()
        
    def open_browser(self, url, serial=None):
        d = self.device(serial)
        d.open_browser(url)
        
    def startScreenRecord(self, serial=None):
        """ 开始录屏并返回一个句柄 """
        d = self.device(serial)
        r = d.screenrecord(no_autostart=True)
        r.start()
        return r
        
    def stopScreenRecord(self, startRecordH, savePath='./video.mp4'):
        """ 停止录屏,startRecordH为录屏句柄 """
        startRecordH.stop_and_pull(savePath)
    
    # def startWifiConnect(self):
    #     # adb tcpip 5555
        
    